import datetime
import typing

from flask_security import SQLAlchemySessionUserDatastore
from sqlalchemy import create_engine
from sqlalchemy.orm import (
    declarative_base,
    scoped_session,
    sessionmaker,
)

Base = declarative_base()


def init_db(path):
    engine = create_engine(f"sqlite:///{path}")
    db_session = scoped_session(
        sessionmaker(autocommit=False, autoflush=False, bind=engine)
    )
    Base.query = db_session.query_property()
    Base.metadata.create_all(bind=engine)

    return db_session


class UserDatastore(SQLAlchemySessionUserDatastore):
    """A UserDatastore implementation that assumes the
    use of
    `Flask-SQLAlchemy <https://pypi.python.org/pypi/flask-sqlalchemy/>`_
    for datastore transactions.
    :param db:
    :param user_model: See :ref:`Models <models_topic>`.
    :param role_model: See :ref:`Models <models_topic>`.
    """

    # pyflakes are a bit picky with F821 (Undefined name),
    # not even sure "Message" should ba considered as an undefined name
    def __init__(
        self,
        *args,
        message_model=typing.Type["Message"],  # noqa: F821
        **kwargs,
    ):
        SQLAlchemySessionUserDatastore.__init__(self, *args, **kwargs)
        self._message_model = message_model
        self._messages_users_model = typing.Type["MessagesUsers"]  # noqa: F821

    def create_message(self, message, from_username, from_nickname, from_host):
        return self.put(
            self._message_model(
                message=message,
                at=datetime.datetime.now(),
                from_username=from_username,
                from_nickname=from_nickname,
                from_host=from_host,
            )
        )

    def add_message_to_user(self, user, message):
        user.messages.append(message)
        self.put(user)
        self.commit()

    def get_all_messages(self):
        return self._message_model.query.all()

    def append_roles(self, user, roles):
        for role in roles:
            if not user.has_role(role):
                if not self.find_role(role):
                    _r = self.create_role(name=role)
                    user.roles.append(_r)
                else:
                    user.roles.append(self.find_role(role))

        self.commit()



import os

from scandir import scandir


def scantree(path, include):
    res = []

    try:
        res = _scantree_rec(path, include, [])
    except OSError:
        pass

    return res


def _scantree_rec(path, include=[], files=[]):
    for entry in scandir(path):
        if entry.is_dir(follow_symlinks=False):
            _scantree_rec(entry.path, include, files)
        elif entry.is_file():
            if os.path.splitext(entry.path)[1][1:] in include:
                files.append(entry.path)

    return files



# -*- coding: utf-8 -*-
import base64

from mxcubecore import HardwareRepository as HWR

from mxcubeweb.core.components.component_base import ComponentBase


class Workflow(ComponentBase):
    def __init__(self, app, config):
        super().__init__(app, config)

    def get_available_workflows(self):
        workflows = {}
        beamline = HWR.beamline

        try:
            for wf in beamline.workflow.get_available_workflows():
                # Rename name and path to wfname and wfpath in order to avoid name
                # clashes
                wf["wfname"] = wf.pop("name")
                wf["wfpath"] = wf.pop("path")

                workflows[wf["wfname"]] = wf
        except Exception:
            pass

        if getattr(beamline, "gphl_workflow", None):
            # Add Global Phasing workflows if available
            workflows.update(beamline.gphl_workflow.get_available_workflows())

        return {"workflows": workflows}

    def submit_parameters(self, params):
        HWR.beamline.workflow.set_values_map(params)

    def update_gphl_parameters(self, params):
        HWR.beamline.emit(params["signal"], params["instruction"], params["data"])

    def get_mesh_result(self, gid, _type="heatmap"):
        base64data = HWR.beamline.sample_view.get_grid_data(gid)
        base64data = base64data if base64data else ""

        data = base64.b64decode(base64data)
        return data

    def test_workflow_dialog(self, wf):
        dialog = {
            "properties": {
                "name": {
                    "title": "Task name",
                    "type": "string",
                    "minLength": 2,
                },
                "description": {
                    "title": "Description",
                    "type": "string",
                    "widget": "textarea",
                },
                "dueTo": {
                    "title": "Due to",
                    "type": "string",
                    "widget": "compatible-datetime",
                    "format": "date-time",
                },
            },
            "required": ["name"],
            "dialogName": "Trouble shooting !",
        }

        return dialog
